{  copyright 1997 by Ian Hickson. Please include my name in your credits if you use this component.
  This is FREEWARE. Please distribute.
  I would appreciate it if you could e-mail me any of your components/programs/source code, etc... as a kind of registration.
  You will then possibly receive upgrades and so on. Note that if you make any modification to this code I ask you to email
  me your change before you distribute it (just so that I know what I contributed towards).  Many Thanks! }

{ Long Live Delphi! }

unit Arrow;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus;

type
  TAngle = (adUp, adLeft, adDown, adRight); { Arrow Direction }

  TArrow = class(TGraphicControl)
  private
    { Private declarations }
    FAngle : TAngle;
    FStalk : Integer;
    FHead : Byte;
    FArrowColor : TColor;
  protected
    { Protected declarations }
    Function GetAbout : String;
    Procedure SetAngle (Value: TAngle);
    Procedure SetStalk (Value: Integer);
    Procedure SetHead (Value: Byte);
    Procedure SetArrowColor (Value: TColor);
    Procedure SetTransparent (Value : Boolean);
    Function GetTransparent : Boolean;
    Procedure SetDummy (Value:String);
  public
    { Public declarations }
    procedure Paint; override;
    constructor Create (AOwner: TComponent); override;
  published
    { Published declarations }
    property About : string read GetAbout write SetDummy stored False;
    property Direction : TAngle read FAngle write SetAngle default adDown;
    property WidthStalk : Integer read FStalk write SetStalk default 1;
    property PercentageHead : Byte read FHead write SetHead default 20;
    property ColorArrow : TColor read FArrowColor write SetArrowColor default clWindowText;
    property Transparent : Boolean read GetTransparent write SetTransparent;
    property DragCursor;
    property DragMode;
    property Color;
    property ParentColor;
    property PopupMenu;
    property ParentShowHint;
    property ShowHint;
    property OnClick;
    property OnDragOver;
    property OnMouseMove;
    property OnDblClick;
    property OnEndDrag;
    property OnMouseUp;
    property OnDragDrop;
    property OnMouseDown;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Extra', [TArrow]);
end;

Function TArrow.GetAbout;
begin
   Result := ' copyright 1997 by Ian Hickson. Please include my name in your credits if you use this component, Thank you. '+
             'This is FREEWARE. Please distribute. See source code for more information.';
end;

Procedure TArrow.SetDummy (Value:String);
begin
   Value := ''; { Basically, nothing }
end;

Procedure TArrow.SetAngle (Value: TAngle);
begin
   If Value <> FAngle then
   begin
      FAngle := Value;
      Invalidate;
   end;
end;

Procedure TArrow.SetStalk (Value: Integer);
begin
   If (Value <> FStalk) and (Value < Width) then
   begin
      FStalk := Value;
      Invalidate;
   end;
end;

Procedure TArrow.SetHead (Value: Byte);
begin
   If (Value <> FHead) and (Value <= 100) and (Value >= 0) then
   begin
      FHead := Value;
      Invalidate;
   end;
end;

Procedure TArrow.SetArrowColor (Value: TColor);
begin
   If Value <> FArrowColor then
   begin
      FArrowColor := Value;
      Invalidate;
   end;
end;

Procedure TArrow.SetTransparent (Value : Boolean);
begin
   If (Value <> Transparent) then
   begin
      If Value then
         ControlStyle := ControlStyle - [csOpaque]
      else
         ControlStyle := ControlStyle + [csOpaque];
      Invalidate;
   end;
end;

Function TArrow.GetTransparent : Boolean;
begin
   Result := not (csOpaque in ControlStyle);
end;

procedure TArrow.Paint;
begin
   Canvas.Pen.Color := Color;
   Canvas.Pen.Style := psClear;
   Canvas.Pen.Width := 0;
   Canvas.Brush.Color := Color;
   If not Transparent then
      Canvas.Rectangle(0,0,Width,Height);
   Canvas.Pen.Color := FArrowColor;
   Canvas.Pen.Width := FStalk;
   Canvas.Pen.Style := psSolid;
   Case FAngle of
    adDown  : begin
                 Canvas.MoveTo(Width div 2, 0);
                 Canvas.LineTo(Width div 2, Round(Height*((100-FHead)/100)));
              end;
    adUp    : begin
                 Canvas.MoveTo(Width div 2, Height);
                 Canvas.LineTo(Width div 2, Height-Round(Height*((100-FHead)/100)));
              end;
    adLeft :  begin
                 Canvas.MoveTo(Width, Height div 2);
                 Canvas.LineTo(Width-Round(Width*((100-FHead)/100)), Height div 2);
              end;
    adRight : begin
                 Canvas.MoveTo(0, Height div 2);
                 Canvas.LineTo(Round(Width*((100-FHead)/100)), Height div 2);
              end;
   end;
   Canvas.Brush.Color := FArrowColor;
   Canvas.Pen.Width := 0;
   If FHead <> 0 then
   Case FAngle of
    adDown    : begin
                   Canvas.Polygon ( [ Point(Width div 2, Height),
                                      Point(0, Round(Height*((100-FHead)/100))),
                                      Point(Width, Round(Height*((100-FHead)/100)))
                                  ] );
                end;
    adRight    : begin
                   Canvas.Polygon ( [ Point(Width, Height div 2),
                                      Point(Round(Width*((100-FHead)/100)), 0),
                                      Point(Round(Width*((100-FHead)/100)), Height)
                                  ] );
                end;
    adUp      : begin
                   Canvas.Polygon ( [ Point(Width div 2, 0),
                                      Point(0, Height-Round(Height*((100-FHead)/100))),
                                      Point(Width, Height-Round(Height*((100-FHead)/100)))
                                  ] );
                end;
    adLeft   : begin
                   Canvas.Polygon ( [ Point(0, Height div 2),
                                      Point(Width-Round(Width*((100-FHead)/100)), 0),
                                      Point(Width-Round(Width*((100-FHead)/100)), Height)
                                  ] );
                end;
   end;
end;

constructor TArrow.Create (AOwner : TComponent);
begin
   inherited Create (AOwner);
   Width := 10;
   Height := 40;
   FHead := 25;
   FStalk := 1;
   FAngle := adDown;
   FArrowColor := clWindowText;
   ControlStyle := ControlStyle - [csOpaque];
end;


end.
